@@ -328,16 +328,11 @@ def enc_apl(bng, x, met):
             elif met == 'int':   xe[dv] = h - 1;          cname_e = cname + '_encINT'
             elif met == 'dv':    xe[:, j] = dv.flatten()*1; cname_e = cname_e + [cname + '_encDV' + str(j + 1)]; j += 1
         if met == 'dv': xe = xe.astype(int)
-        print('isnan(xe): ', np.any(np.isnan(xe)))
-        if np.any(np.isnan(xe)):
-            prinr("DA EBA MAMA TI BEEEEEEEEEEEEE!!!!!!!!!!!!!!!!!")
         xenc = np.hstack((xenc, xe))
         if isinstance(cname_e, str): ss.append(cname_e)
         else:                        ss = ss + cname_e
-
     return xenc, ss
 
-
 ###################################################################################
 def inv_distr(DistrType, pval, df1, df2, x1, x2, tol):
     ITMAX = 100;
@@ -662,7 +657,7 @@ def sbng(bng, md=5, skip_mos=['m', 'o', 'sv'], met_dist='manh', met='co1y', nmin
         else:
             bns = bng[i]['bns'];    cname = bng[i]['cname'];    xtp = bng[i]['xtp'];    ytp = bng[i]['ytp']
         bns, bns_1 = skipmosbns(bns, skip_mos)
-
+        xtp1 = xtp
         if xtp == 'nom':
             [n, nw, syy, syy2, wy] = bns2arr(bns)
             [x, w] = mdep(syy, nw, wy)
@@ -670,20 +665,26 @@ def sbng(bng, md=5, skip_mos=['m', 'o', 'sv'], met_dist='manh', met='co1y', nmin
             bins = {}
             for j in range(len(ix)): bins[j + 1] = bns[ix.flatten()[j]]
             bns = bins
-            xtp = 'ord'
+            xtp1 = 'ord'
 
         if met == 'hclust':
-            bns, D = spih_sb_hclust(bns, xtp, md, met_dist)
+            bns, D = spih_sb_hclust(bns, xtp1, md, met_dist)
         elif met == 'co1y':
             gd = nmin_uy[0]
             bd = nmin_uy[1]
-            bns, D = spih_sb_co1y(bns, xtp, ytp, md, gd, bd, mar, pt, ba, dGBInd, tolmindGBInd, mt, minB, tolminB, maxprm)
-
+            bns, D = spih_sb_co1y(bns, xtp1, ytp, md, gd, bd, mar, pt, ba, dGBInd, tolmindGBInd, mt, minB, tolminB, maxprm)
+        flg = 0
         if bns is None:
-            sb[i]['bns'] = {}
+            bns = bng[i]['bns']
+            for j in range(len(bns), 1, -1):
+                if bns[j]['type'] == 'Normal' and bns[j - 1]['type'] == 'Normal':
+                    bns = mrgbns(bns, j-1)
+            sb[i]['bns'] = bns
+            flg = 1
         else:
             sb[i]['bns'] = bns
-        for j in range(len(bns_1)): sb[i]['bns'][len(sb[i]['bns']) + 1] = bns_1[j + 1]
+        if not flg:
+            for j in range(len(bns_1)): sb[i]['bns'][len(sb[i]['bns']) + 1] = bns_1[j + 1]
         sb[i]['cname'] = bng[i]['cname']
         sb[i]['xtp'] = xtp
         sb[i]['ytp'] = ytp
@@ -893,7 +894,7 @@ def spih(bns, xtp, md, nd, met):
             bns = mrgbns(bns, m - 1)
         else:
             c = []
-            for k in range(1, len(bns) + 1): c.append(bns[k]['nw'])
+            for k in range(1, len(bns)): c.append(bns[k]['nw'])
             ni2 = bns[len(bns)]['nw'] + c
             i = np.argmin(ni2) + 1
             bns = mrgbns(bns, i, m)
