@@ -5,7 +5,7 @@ Matlab version: 26 Apr 2009
 Course: Multivariable Control Systems
 """
 import numpy as np
-from sf import *
+from aislab.gnrl.sf import *
 ####################################################################################
 # def trnd():
 ####################################################################################
