@@ -107,7 +107,6 @@ def dmpm(U=np.empty((0, 0)),
 
 
 ##############################################################################
-# def dmpv(Y, i, j, nn, r=0, par={'na':0, 'nb':0, 'nc':0, 'pm0':False}):
 def dmpv(U=np.empty((0, 0)), Y=np.empty((0, 0)), E=np.empty((0, 0)),
          na=None,
          nb=None,
@@ -198,8 +197,6 @@ def dmpv(U=np.empty((0, 0)), Y=np.empty((0, 0)), E=np.empty((0, 0)),
             Fi[range(i, (N - n) * r, r)] = MM
             F = np.hstack((F, Fi))
         return F
-
-
 ##############################################################################
 # todo: make dv2dm() faster - skip for->for
 def dv2dm(x, cols):
@@ -214,8 +211,6 @@ def dv2dm(x, cols):
             X[row][column] = x[current_pos]
             current_pos += 1
     return X
-
-
 ##############################################################################
 def elspm(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcnv=None, fcnv=1e-8, gcnv=None,
           dsp=False):
@@ -223,15 +218,11 @@ def elspm(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcn
     n = max(na, nb, nc)
     nn = n - max(na, nb)
 
-    mdl = lspm(U, Y, na, nb, pm0)
-    Pmab = mdl['Pm']
-    # Fab = dmpm(U[nn:], Y[nn:], par=par)
+    Pmab = lspm(U, Y, na, nb, pm0)
     Fab = dmpm(U[nn:], Y[nn:], na=na, nb=nb, nc=0, pm0=pm0)
     E = np.vstack((np.zeros([n, r]), Y[n:, :] - Fab@Pmab))
     f = np.sum(np.diag(E.T@E) / N)
-    if nc == 0:
-        mdl = {'Pm': Pmab, 'par': par}
-        return mdl, E
+    if nc == 0: return Pm, E
     Pm = np.vstack((Pmab, np.zeros([r * nc, r])))
     itr = 0
     iterate = 1
@@ -254,23 +245,13 @@ def elspm(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcn
     if dsp == True:
         print("It took [elspm.py]", itr, "iterations to reach the end condition.")
         print(maxiter, "is the maximum number of iterations.")
-    par = {'na': na, 'nb': nb, 'nc': nc, 'pm0': pm0}
-    opt = {'maxiter': maxiter, 'axcnv': axcnv, 'xcnv': xcnv, 'afcnv': afcnv, 'fcnv': fcnv, 'gcnv': gcnv, 'dsp': dsp}
-    mdl = {'Pm': Pm, 'par': par, 'opt': opt}
-    return mdl, E
-
-
+    return Pm, E
 ##############################################################################
-def elspm_apl(U, Y, E, mdl):
-    Pm = mdl['Pm']
-    par = mdl['par']
-    F = dmpm(U, Y, E, par=par)
+def elspm_apl(U, Y, E, Pm, na=0, nb=0, nc=0, pm0=0):
+    F = dmpm(U, Y, E, na, nb, nc, pm0)
     Ym = np.dot(F, Pm)
     return Ym
-
-
 ##############################################################################
-
 def elspv(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcnv=None, fcnv=1e-8, gcnv=None,
           dsp=False):
     N, r = Y.shape
@@ -292,14 +273,14 @@ def elspv(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcn
     n = int(np.max(np.hstack((na, nb, nc))))
     nn = int(n - np.max(np.hstack((na, nb))))
     arx = lspv(U, Y, na=na, nb=nb, pm0=pm0)
-    Ymab = lspv_apl(U[nn:], Y[nn:], arx)
+    Ymab = lspv_apl(U[nn:], Y[nn:], arx, na=na, nb=nb, pm0=pm0)
     if nc.size == 0:
         E = np.empty((0, 0))
         return arx, E
     Fab = dmpv(U[nn:], Y[nn:], na=na, nb=nb, nc=np.zeros((nc.shape)), pm0=pm0)
     E = np.vstack((np.zeros([n, r]), Y[n:] - Ymab))
     f = np.sum(np.diag(E.T@E) / N)
-    pm = np.vstack((arx['pm'], np.zeros((int(np.sum(nc)), 1))))
+    pm = np.vstack((arx, np.zeros((int(np.sum(nc)), 1))))
     Pm = np.vstack((pv2m(pm, na=na, nb=nb, pm0=pm0), np.zeros((r * int(np.max(nc)), r))))
     itr = 0
     iterate = 1
@@ -321,23 +302,15 @@ def elspv(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcn
             pm = (pm + pm_1) / 2
             if dsp:
                 print('iter: ', itr, ' Reduce step size!')
-        iterate, msg = stopcrt(x=Pm, F=f, xx=Pm_1, FF=f_1, itr=itr, par={'maxiter': 50, 'xcnv': 1e-8, 'fcnv': 1e-8})
-    par = {'na': na, 'nb': nb, 'nc': nc, 'pm0': pm0}
-    opt = {'maxiter': maxiter, 'axcnv': axcnv, 'xcnv': xcnv, 'afcnv': afcnv, 'fcnv': fcnv, 'gcnv': gcnv, 'dsp': dsp}
-    mdl = {'pm': pm, 'par': par, 'opt': opt}
-    mdl['pm'] = pm1 if f > f_1 else pm
-    return mdl, E
-
-
+        iterate, msg = stopcrt(x=Pm, F=f, xx=Pm_1, FF=f_1, itr=itr, maxiter=50, xcnv=1e-8, fcnv=1e-8)
+    pm = pm1 if f > f_1 else pm
+    return pm, E
 ##############################################################################
-def elspv_apl(U, Y, E, mdl):
-    par = mdl['par']
-    Pm = pv2m(mdl['pm'], na=par['na'], nb=par['nb'], nc=par['nc'], pm0=par['pm0'])
-    F = dmpm(U, Y, E, na=par['na'], nb=par['nb'], nc=par['nc'], pm0=par['pm0'])
+def elspv_apl(U, Y, E, pm, na=0, nb=0, nc=0, pm0=0):
+    F = dmpm(U, Y, E, na, nb, nc, pm0)
+    Pm = pv2m(pm, na, nb, nc, pm0)
     Ym = F@Pm
     return Ym
-
-
 ##############################################################################
 # glspm():
 # glspm_apl():
@@ -421,17 +394,11 @@ def lspm(U, Y, na=0, nb=0, pm0=False, w=None):
         else:
             W = np.diag(w[n:N])
             Pm = nsinv(F.T@W@F)@F.T@W@Y[n:N][:]
-    par = {'na': na, 'nb': nb, 'pm0': pm0}
-    mdl = {'Pm': Pm, 'par': par}
-    return mdl
+    return Pm
 
 
 ##############################################################################
-def lspm_apl(U, Y, mdl):
-    Pm = mdl['Pm']
-    na = mdl['par']['na']
-    nb = mdl['par']['nb']
-    pm0 = mdl['par']['pm0']
+def lspm_apl(U, Y, Pm, na=0, nb=0, pm0=0):
     if max(na, nb) == 0:
         if pm0: U = np.hstack((np.ones((U.shape[0], 1)), U))
         Ym = U@Pm
@@ -469,29 +436,16 @@ def lspv(U=np.empty((0, 0)),
         pm = np.empty((0, 0))
     else:
         pm = nsinv(F.T@F)@F.T@vec(Y[n:].T)
-    par = {'na': na, 'nb': nb, 'pm0': pm0}
-    mdl = {'pm': c_(pm), 'par': par}
-    return mdl
-
-
+    return c_(pm)
 ##############################################################################
-def lspv_apl(U, Y, mdl):
-    pm = mdl['pm']
-    na = mdl['par']['na']
-    nb = mdl['par']['nb']
-    pm0 = mdl['par']['pm0']
-    Pm = pv2m(pm, na=na, nb=nb, pm0=pm0)
+def lspv_apl(U, Y, pm, na=None, nb=None, pm0=0):
+    Pm = pv2m(pm, na, nb, pm0=pm0)
     F = dmpm(U, Y, na=na, nb=nb, pm0=pm0)
     Ym = F@Pm
     return Ym
-
-
 ##############################################################################
-def pm2v(Pm, par={'na': 0, 'nb': 0, 'nc': 0, 'pm0': 0}):
-    # todo: include pm0
-    # todo: include nc
-    na = par['na']
-    nb = par['nb']
+def pm2v(Pm, na=None, nb=None, nc=0, pm0=0):
+    # todo: include nc and pm0
     r = m = 1
     z = na + nb
     if z == 0: return np.empty((1, 1))
@@ -507,8 +461,6 @@ def pm2v(Pm, par={'na': 0, 'nb': 0, 'nc': 0, 'pm0': 0}):
                 pij = pari[j + j + r * na:m * nb + j + r * na:m]
                 pm = np.hstack((pm, pij))
     return c_(pm)
-
-
 ##############################################################################
 def pv2m(pm,
          na=None,
@@ -522,7 +474,7 @@ def pv2m(pm,
     elif not np.array([na == None]).any():
         r = na.shape[0]
         m = 0
-    else:
+    elif not np.array([nc == None]).any():
         r = nc.shape[0]  # todo: use try-except --> at least one of {na, nb, nc} should be provided
         m = 0
     if np.array([na == None]).any():
@@ -576,8 +528,6 @@ def pv2m(pm,
                     nc[i, j] + ii), 0]
                 ii += nc[i, j]
     return Pm.T
-
-
 ##############################################################################
 # function Y = repmatc(X, m)
 # % Columnwise reproduction (m times) of X matrix, i.e. Y = [X X ... X]
@@ -614,7 +564,6 @@ def roblspm(U, Y, na=0, nb=0, pm0=0, maxiter=50, dvaf=1e-2, hst=0):
     N, r = Y.shape
     m = U.shape[1]
     n = max([na, nb])
-    par = {'na': na, 'nb': nb, 'pm0': pm0}
     F = dmpm(U, Y, na=na, nb=nb, nc=0, pm0=pm0)
     Pm = nsinv(F.T@F)@F.T@Y[n:N][:]
     Ym = F@Pm
@@ -624,7 +573,7 @@ def roblspm(U, Y, na=0, nb=0, pm0=0, maxiter=50, dvaf=1e-2, hst=0):
     if hst:
         VAFw = vafw
         VAF = vaf0
-        PM = pm2v(Pm, par)
+        PM = pm2v(Pm, na=na, nb=nb, pm0=0) # todo: make pm2v() to work for pm0=1
     iter_n = 0
     iterate = 1
     while iterate:
@@ -638,29 +587,20 @@ def roblspm(U, Y, na=0, nb=0, pm0=0, maxiter=50, dvaf=1e-2, hst=0):
         vafw = vaf(Y1, Ym, ww)
         vaf0 = vaf(Y1, Ym)
         if hst:
-            PM = np.append(PM, pm2v(Pm, par)) if PM.size > 0 else pm2v(Pm, par)  # <<<<<
+            PM = np.append(PM, pm2v(Pm, na, nb, pm0=0)) if PM.size > 0 else pm2v(Pm, na, nb, pm0=0)
             VAFw = np.append(VAFw, vafw)
             VAF = np.append(VAF, vaf0)
         iterate = 0 if any(abs(vafw - vafw_1) <= dvaf) or iter_n >= maxiter else 1
     if hst:
-        st = dict(PM=PM, vafw=vafw, vaf0=vaf0)
+        st = {'PM':PM, 'vafw':vafw, 'vaf0':vaf0, 'iterations':iter_n}
     else:
         st = []
-    mdl = dict(Pm=Pm, par=par, st=st)
-    return mdl
-
-
+    return Pm, st
 ##############################################################################
-def roblspm_apl(U, Y, mdl):
-    Pm = mdl['Pm']
-    na = mdl['par']['na']
-    nb = mdl['par']['nb']
-    pm0 = mdl['par']['pm0']
+def roblspm_apl(U, Y, Pm, na=0, nb=0, pm0=0):
     F = dmpm(U, Y, na=na, nb=nb)
     Ym = F@Pm
     return Ym
-
-
 ##############################################################################
 def roblspv(U, Y, na=0, nb=0, pm0=0, maxiter=50, dvaf=1e-2, hst=0):
     N, r = Y.shape
@@ -714,14 +654,11 @@ def roblspv(U, Y, na=0, nb=0, pm0=0, maxiter=50, dvaf=1e-2, hst=0):
         if any(abs(vafw - vafw_1)) <= dvaf or iter_n >= maxiter:
             iterate = False
     if hst:  # st.
-        st = dict(PM=PM, vafw=VAFw, vaf0=VAF, YM=YM)
+        st = {'PM':PM, 'vafw':vafw, 'vaf0':vaf0, 'iterations':iter_n}
     else:
         st = []
-    par = dict(na=na, nb=nb, pm0=pm0)
-    mdl = dict(par=par, pm=pm, st=st, iterations=iter_n)
     # ----------------------------------------------------------------
-    return mdl
-
+    return pm, st
 ##############################################################################
 # defv roblspv_apl():
 ##############################################################################
