@@ -40,14 +40,14 @@ def dspl(flg, **kwargs):
                 print('Stopping Rule (MaxIter = ', kwargs['maxiter'], ') satisfied. /', itr, '/')
             elif 'C' == msg:
                 print('Convergence criterion (FCONV = ', kwargs['fcnv'], ') satisfied. /',
-                      np.abs(F - FF[0])/(np.abs(FF[0]) + 1e-6), '/')
+                      np.abs(F - FF[0])/np.max(np.abs(FF[0]), initial=1e-6), '/')
             elif 'D' == msg:
                 print('Convergence criterion (ABSFCONV = ', kwargs['afcnv'], ') satisfied. /', np.abs(F - FF[0]), '/')
             elif 'E' == msg:
                 print('Convergence criterion (XCONV = ', kwargs['xcnv'], ') satisfied. /', (x - xx[:, 0]).T/x, '/')
             elif 'F' == msg:
                 print('Convergence criterion (GCONV = ', kwargs['gcnv'], ') satisfied. /',
-                      -g.T@H@g/(np.abs(F) + 1e-6), '/')
+                      -g.T@H@g/np.max(np.abs(F), initial=1e-6), '/')
             elif 'G' == msg:
                 print('Termination: reaching the uncretainty level...')
             elif 'H' == msg:
@@ -98,7 +98,7 @@ def f_exp1(args):
 
     return F, args
 ###################################################################################
-def f_exp2(args):
+def f_expW(args):
    # F for exponential model: ym = a + b*exp(c*x)
     h = args['data'].shape[1]
     x = args['data'][:, np.arange(0, h - 3)]
@@ -106,18 +106,24 @@ def f_exp2(args):
     w = c_(args['data'][:, -2])
     pm = args['x']
     if x.shape[1] == 0: x = c_(x)
-    ym = exp2_apl(x, pm)
+    ym = apl_expW(x, pm)
     e = y - ym
     F = e.T@(e*w)
     args['data'][:, -1] = ym.flatten()
 
-    b = 0.01
-    c = 1e6
+    # print(np.hstack((e, w)))
+
+    b = 1e-8
+    c = 1e3
     d1 = pm.flatten()
     d2 = (1 - (pm[0, 0] + pm[1, 0]))**2*c
     A = np.diag((d1 < b))*c
     F = F + pm.T@A@pm + d2
 
+    print('F:       ', F)
+    print('pmApm:   ', pm.T@A@pm)
+    print('1-(a+b): ', d2)
+
     return F, args
 ###################################################################################
 def f_lgr(args):
@@ -183,6 +189,29 @@ def f_log1(args):
 
     return F, args
 ###################################################################################
+def f_logW(args):
+    # F for exponential model: ym = a + b*ln(c*x + 1)
+    pm = args['x']
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    y = c_(args['data'][:, -3])
+    w = c_(args['data'][:, -2])
+    if x.shape[1] == 0: x = c_(x)
+    ym = apl_logW(x, args['x'])
+    e = y - ym
+    F = e.T@(e*w)
+    args['data'][:, -1] = ym.flatten()
+
+    b = 0.01
+    c = 1e6
+    d1 = pm.flatten()
+    d2 = (pm[0, 0] - 1)**2*c
+
+    A = np.diag((d1 < b))*c
+    F = F + pm.T@A@pm + d2
+
+    return F, args
+###################################################################################
 def g_exp(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
@@ -236,7 +265,7 @@ def g_exp1(args=None):
 
     return g
 ###################################################################################
-def g_exp2(args=None):
+def g_expW(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
     # Date:   20.12.2009
@@ -257,7 +286,7 @@ def g_exp2(args=None):
     g[2] = (-2*pm[1]*(w*e*x).T@tmp)[0]
 
     b = 0.01
-    c = 1e6
+    c = 1e3
     dd = pm.flatten()
     A = np.diag((dd < b))*c
     g = g + 2*A@pm + np.array([[-2*(1 - (pm[0,0] + pm[1,0]))*c], [-2*(1 - (pm[0,0] + pm[1,0]))*c], [0]])
@@ -304,6 +333,33 @@ def g_log(args=None):
 
     return g
 ###################################################################################
+def g_logW(args=None):
+    # ---------------------------------------------
+    # Author: Alexander Efemov
+    # Date:   20.12.2009
+    # Course: Modelling and Processes Optimization
+    # ---------------------------------------------
+    # gradient of F for logistic regression model: ym = 1/(1 + exp(-x*pm))
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    y = c_(args['data'][:, -3])
+    w = c_(args['data'][:, -2])
+    ym = c_(args['data'][:, -1])
+    pm = args['x']
+    e = y - ym
+    g = np.zeros((len(pm), 1))
+    g[0] = (-2*w.T@e)[0, 0]
+    g[1] = (-2*(w*e).T@np.log(abs(pm[2]*x) + 1))[0]
+    g[2] = ((-2*pm[1]*(w*e/(abs(pm[2]*x) + 1)).T@x)*np.sign(pm[2]*x + 1))[0]  # !!!: np.sign(0) = 0
+
+    b = 1e-8
+    c = 1e6
+    dd = pm.flatten()
+    A = np.diag((dd < b))*c
+    d2 = (pm[0, 0] - 1)**2*c
+    g = g + 2*A@pm + np.array([[2*(pm[0,0] - 1)*c], [0], [0]])
+    return g
+###################################################################################
 def h_exp(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
@@ -366,7 +422,7 @@ def h_exp1(args=None):
     dd = pm.flatten()
     A = np.diag((dd < b))*c
 ###################################################################################
-def h_exp2(args=None):
+def h_expW(args=None):
     # ---------------------------------------------
     # Author: Alexander Efemov
     # Date:   20.12.2009
@@ -393,8 +449,8 @@ def h_exp2(args=None):
     H[2, 2] = 2*pm[1]**2*((w*tmp*x**2).T@tmp)[0] - 2*pm[1]*((w*e*x**2).T@tmp)[0]
 
     pm = args['x']
-    b = 0.01
-    c = 1e6
+    b = 1e-8
+    c = 1e3
     dd = pm.flatten()
     A = np.diag((dd < b))*c
     H = H + 2*A + np.array([[2*c, 2*c, 0], [2*c, 2*c, 0], [0, 0, 0]])
@@ -446,6 +502,40 @@ def h_log(args=None):
     H = H + 2*A
     return H
 ###################################################################################
+def h_logW(args=None):
+    # ---------------------------------------------
+    # Author: Alexander Efemov
+    # Date:   20.12.2009
+    # Course: Modelling and Processes Optimization
+    # ---------------------------------------------
+    # Hessian of F for logistic regression model: ym = 1/(1 + exp(-x*pm))
+    h = args['data'].shape[1]
+    x = args['data'][:, np.arange(0, h - 3)]
+    w = c_(args['data'][:, -2])
+    y = c_(args['data'][:, -3])
+    ym = c_(args['data'][:, -1])
+    pm = args['x']
+    e = y - ym
+    H = np.zeros(shape=(3, 3))
+    tmp = np.log(abs(pm[2]*x + 1))
+    H[0, 0] = 2*np.sum(w)
+    H[0, 1] = 2*(w.T@tmp)[0]
+    H[0, 2] = (2*pm[1]*((w/abs(pm[2]*x + 1)).T@x)*np.sign(pm[1]*x + 1))[0]
+    H[1, 0] = H[0, 1]
+    H[1, 1] = 2*((w*tmp).T@tmp)[0]
+    H[1, 2] = (2*pm[1]*((w*tmp/abs(pm[2]*x + 1)).T@x)*np.sign(pm[1]*x + 1) - 2*((w*e/abs(pm[2]*x + 1)).T@x)*np.sign(pm[1]*x + 1))[0]
+    H[2, 0] = H[0, 2]
+    H[2, 1] = H[1, 2]
+    H[2, 2] = 2*pm[1]**2*((w/abs(pm[2]*x + 1)*x).T@x)[0] - 2*pm[1]*((w*e*x/abs(pm[2]*x + 1)).T@x)[0]
+
+    pm = args['x']
+    b = 0.01
+    c = 1e6
+    dd = pm.flatten()
+    A = np.diag((dd < b))*c
+    H = H + 2*A + np.array([[2*c, 0, 0], [0, 0, 0], [0, 0, 0]])
+    return H
+###################################################################################
 def exp_apl(x, pm):
     ym = pm[0] + pm[1]*np.exp(x)
     return ym
@@ -454,7 +544,7 @@ def exp1_apl(x, pm):
     ym = pm[0] + pm[1]*np.exp(x + pm[2])
     return ym
 ###################################################################################
-def exp2_apl(x, pm):
+def apl_expW(x, pm):
     ym = pm[0] + pm[1]*np.exp(x*pm[2])
     return ym
 ###################################################################################
@@ -473,6 +563,10 @@ def log1_apl(x, pm):
     ym = pm[0] + pm[1]*np.log(pm[2]*abs(x) + 1)
     return ym
 ###################################################################################
+def apl_logW(x, pm):
+    ym = pm[0] + pm[1]*np.log(abs(pm[2]*x + 1))
+    return ym
+###################################################################################
 def optOmit(x, xx, F, FF, g, g_1, H, H_1, Hinv, Hinv_1, iterate, s, tmp, sc, smeth, dsp_op):
     # ---------------------------------------------
     # Author: Alexander Efemov
